/*------------------------------------------------------------------------------
 *       Filename:  ar.h
 *
 *    Description:  Header file for activity recognition
 *
 *        Version:  4.0
 *        Created:  05/07/2012 06:46:23 PM
 *       Revision:  none
 *       Compiler:  gcc
 *
 *        Authors:  Diane J. Cook, cook@eecs.wsu.edu
 *   Organization:  Washington State University
------------------------------------------------------------------------------*/

#ifndef AR_H
#define AR_H

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <ctype.h>
#include "sys/stat.h"

#define MAXSTR 80          // Maximum string size
#define BUFFERSIZE 50000
#define TRAIN 0
#define TEST 1
#define NONE 0
#define TIMEWEIGHT 1
#define MUTUALINFO 2
#define MUTUALINFO_TIME 3
#define NUMWINDOWS 4
#define MAXWINDOW 30
#define LN 2.30265441084134
#define MIN 0.0000001
#define KAPPA 0.125
#define TRUE 1
#define FALSE 0
#define HOUR 0
#define SENSOR1 1
#define SENSOR2 2
#define SECONDS 3
#define EQ 0
#define LT 1
#define GE 2
#define MINNUMOBJ 2
#define C45 TRUE
#define WRITEDATA FALSE
#define CF 0.25
#define PRUNE FALSE
#define TREE 0
#define LEAF 1

#define min(a,b) (((a) < (b)) ? (a) : (b))
#define max(a,b) (((a) > (b)) ? (a) : (b))

/* Decision tree node */
typedef struct TreeNode
{
   int attribute;
   float threshold;
   int numChildren;
   struct TreeNode **children;
   int Class;
   int numInstances;
   int *instances;
} TreeNode, *TreeNodePtr;


char **filenames = NULL;   // names of the training data files
char *modelfname = NULL;   // filename for storing / loading the activity model
int Mode = TRAIN;          // 0=train, 1=test
int Weight = NONE;         // 0=none, 1=time, 2=mutual info, 3=time+mutual info
int OutputLevel = 1;       // Ranges from 0 to 2
int NumFiles = 0;          // number of training data files
int IgnoreOther = FALSE;   // TRUE=ignore OtherActivity

// AR variables
char **sensornames = NULL; // sensor names
char **activitynames = NULL; // activity names
int NumActivities = 0;     // total number of activities
int NumData = 0;           // total number of data samples
int NumSensors = 0;        // total number of sensors
int windata[MAXWINDOW][4];
int wincnt = 0;
int PrevActivity = 0;
int CurActivity = 0;
int *labels = NULL;
int numright = 0;
int *classfreq = NULL;
int *attused = NULL;
int *sensortimes = NULL;   // time since sensor last fired
float **data = NULL;       // feature vector matrix derived from sensor data
float **MI = NULL;         // mutual information for sensor pairs

int NumFeatures;            // feature vector size

// Dynamic window variables
int **actwind = NULL;
int *factwind = NULL;
int **senact = NULL;
int *fsenact = NULL;

// Dominant sensor ids for previous three windows
int numwin = 0;
int dominant = 0;
int prevwin1 = 0;
int prevwin2 = 0;

float Val[9] = {  0,  0.001, 0.005, 0.01, 0.05, 0.10, 0.20, 0.40, 1.00};
float Dev[9] = {4.0,  3.09,  2.58,  2.33, 1.65, 1.28, 0.84, 0.25, 0.00};

void ReadConfig(char *name);  // Read the configuration file
void PrintParms();            // Print the program parameter values
void ComputeProbActWind();    // Compute the activity probability distributions
                              // for alternative sensor windows
void ComputeProbSenAct();     // Compute the activity probability distribution
                              // for each sensor
void ComputeMI();             // Compute mutual information for sensor pairs
void ReadData();              // Read the data from the individual files and
                              // transform data into feature vectors
                              // Update current window, create feature vector
void TrainModel();            // Learn the activity model
void TestModel();             // Test activity model using data from a file
void SaveParams();            // Save the training parameters
void FreeMemory();            // Free allocated memory
void ComputeNumFeatures();
void DTL();                   // Build a decision tree
void SaveDT(TreeNodePtr tree);
void WriteDT(FILE *fpmodel, TreeNodePtr tree);
void FreeTree(TreeNodePtr tree);

char *TrimStr(char *buffer);  // Trim white space at the end of the string
int FindActivity(char *name); // Find the index of an activity from
                              // an activity list
int FindSensor(char *name);   // Find the index of a sensor from a sensor list
int ComputeFeature(char *time, char *sensorid1, char *sensorid2, float *tempdata);
int ComputeSeconds(char *time); // Compute seconds past midnight
int ComputeHour(char *time);  // Compute hour of day based on 24 hour clock
int GetWindow(char *name);    // Compute best window size for the given sensor
int ReadString(char *str, int pos, char *arg, int length);
                              // Return majority class label
int MajorityClass(int *training, int num, int *small);
int CompactInst(int *training, int numtrain, int att, float val);
int AllOneClass(int *training, int numtrain);
int AttMajorClass(int *training, int numtrain, int att, float val, int test);
int DTLClassify(int instance, TreeNodePtr tree);
int SelectBestAttribute(TreeNodePtr dtnode, int *training, int numtrain,
                        int *attused);
int SelectC45Attribute(TreeNodePtr dtnode, int *training, int numtrain,
                        int *attused);
int FindClass(int *training, int numtrain, int att, float value);
int NumInClass(TreeNodePtr tree, int majorclass, int type);
float *ComputeThresholds(int *training, int numtrain, int att, 
                         int *numthresholds);
float *ComputeC45Thresholds(int *training, int numtrain, int att, 
                         int *numthresholds);
float Entropy(int *freq, int total);
float OldEnt(int *training, int numtrain);
float NewEnt(int *training, int numtrain, int att, float val);
float AddErr(float n, float e);

TreeNodePtr BuildTree(int *training, int numtrain, int parentclass,
   int *attused, int numatt, int small);
TreeNodePtr AllocateTreeNode();
TreeNodePtr LoadModel();             // Load model from a file
TreeNodePtr ReadDT(FILE *fpmodel);
TreeNodePtr PruneTree(TreeNodePtr tree);

#endif // AR_H
